#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <vector>

#include "object.h"
#include "mesh.h"
#include "scene.h"

#include "opengl/opengl.h"
#include "vulkan/vk.h"

/* static functions */

static bool init();
static void cleanup();
static void display();

/* glfw callbacks */
static void key_clbk(GLFWwindow *win, int key, int scancode, int action, int mods);

/* global variables */
bool use_vulkan;
GLFWwindow *win;

/* variables */
static Scene scene;

int main(int argc, char **argv)
{
	for(int i=0; i<argc; ++i) {
		if(strcmp(argv[i], "-opengl") == 0) {
			use_vulkan = false;
			printf("Backend: OpenGL.\n");
		}
		else if(strcmp(argv[i], "-vulkan") == 0) {
			use_vulkan = true;
			printf("Backend: Vulkan.\n");
		}
		else {
			printf("No backend specified. Using OpenGL.\n");
		}
	}

	if(!init()) {
		fprintf(stderr, "Failed to initialize program.\n");
		return 1;
	}

	glfwSetKeyCallback(win, key_clbk);

	while(!glfwWindowShouldClose(win)) {
		display();

		glfwSwapBuffers(win);
		glfwPollEvents();
	}

	atexit(cleanup);
	return 0;
}

static bool init()
{
	if(use_vulkan) {
		if(!init_vulkan())
			return false;
	}
	else {
		if(!init_opengl())
			return false;
	}

	if(!scene.load("data/spot/spot_control_mesh.obj")) {
		fprintf(stderr, "Failed to load scene.\n");
		return false;
	}

	for(size_t i=0; i<scene.objects.size(); ++i) {
		printf("object: %d\n", (int)i);
		printf("mesh: %s\n", scene.objects[i]->mesh->name.c_str());
		printf("material: %s\n", scene.objects[i]->material->name.c_str());
		printf("transform:\n");
		scene.objects[i]->transform.print();
	}
	return true;
}

static void cleanup()
{
	if(use_vulkan) {
		cleanup_vulkan();
	}
	else {
		cleanup_opengl();
	}
}

static void key_clbk(GLFWwindow *win, int key, int scancode, int action, int mods)
{
	if(key == GLFW_KEY_ESCAPE && action == GLFW_PRESS) {
		glfwSetWindowShouldClose(win, GLFW_TRUE);
	}
}

static void display()
{
	if(use_vulkan) {
		display_vulkan();
	}
	else {
		display_opengl();
	}
}
